@@ -14,7 +14,7 @@ from fastmcp.client.transports import StreamableHttpTransport
 from agentmake.plugins.uba.lib.BibleBooks import BibleBooks
 from agentmake.plugins.uba.lib.BibleParser import BibleVerseParser
 from agentmake import agentmake, getOpenCommand, getDictionaryOutput, edit_file, edit_configurations, extractText, readTextFile, writeTextFile, getCurrentDateTime, AGENTMAKE_USER_DIR, USER_OS, DEVELOPER_MODE, DEFAULT_TEXT_EDITOR
-from agentmake.utils.files import searchFolder, isExistingPath
+from agentmake.utils.files import searchFolder, isExistingPath, sanitize_filename
 from agentmake.etextedit import launch_async
 from agentmake.utils.manage_package import getPackageLatestVersion
 from rich.console import Console
@@ -31,7 +31,7 @@ if not USER_OS == "Windows":
     import readline  # for better input experience
 
 # set window title
-set_title(f"圣经研读小伙伴 BibleMate AI")
+set_title("圣经研读小伙伴 BibleMate AI")
 
 parser = argparse.ArgumentParser(description = f"""BibleMate AI {BIBLEMATE_VERSION} CLI options""")
 # global options
@@ -206,16 +206,18 @@ def display_info(console, info, title=None, border_style=config.color_info_borde
     console.print(info_panel)
     console.print()
 
-def backup_conversation(messages, master_plan, console=None, storage_path=None):
+def backup_conversation(messages, master_plan, console=None, storage_path=None, title=None):
     """Backs up the current conversation to the user's directory."""
     if len(messages) > len(DEFAULT_MESSAGES) and ((not console) or (console and storage_path) or (console and not storage_path and config.backup_required)):
         # determine storage path
         if not storage_path:
             if console:
                 timestamp = getCurrentDateTime()
+                if title:
+                    timestamp += "_"+sanitize_filename(title)[:50].replace(" ", "_")
                 storage_path = os.path.join(BIBLEMATE_USER_DIR, "chats", timestamp)
             else:
-                storage_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), "temp")
+                storage_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), f"temp_{config.backend}")
         # create directory if not exists
         if not os.path.isdir(storage_path):
             Path(storage_path).mkdir(parents=True, exist_ok=True)
@@ -234,8 +236,7 @@ def backup_conversation(messages, master_plan, console=None, storage_path=None):
             console.save_html(html_file, inline_styles=True, theme=MONOKAI)
         # Inform users of the backup location
         if console:
-            info = f"已经备份到： {storage_path}\n报告已存档： {html_file}"
-            display_info(console, info)
+            display_info(console, storage_path, title="备份")
 
 def get_border_style():
     if config.agent_mode:
@@ -354,7 +355,7 @@ async def main_async():
         template_pattern = "|".join(template_list)
         template_pattern = f"""^({template_pattern})"""
 
-        user_request = ""
+        original_request = user_request = ""
         master_plan = ""
         messages = deepcopy(DEFAULT_MESSAGES) # set the tone
 
@@ -405,6 +406,27 @@ async def main_async():
                     task.cancel()
                     await asyncio.sleep(0) # Allow cancellation to propagate
                     raise # Re-raise CancelledError
+            # gnerate title
+            async def generate_title():
+                nonlocal console, original_request
+                if not original_request:
+                    return ""
+                generated_title_output = ""
+                generated_title = ""
+                async def run_prompt_engineering():
+                    nonlocal generated_title_output, generated_title
+                    generated_title_output = agentmake(original_request, system=get_system_generate_longer_title(), **AGENTMAKE_CONFIG)
+                    if generated_title_output:
+                        generated_title = generated_title_output[-1].get("content", "").strip().replace("标题: ", "")
+                try:
+                    await thinking(run_prompt_engineering, "撰写标题中 ...")
+                    if not generated_title_output:
+                        display_cancel_message(console)
+                except (KeyboardInterrupt, asyncio.CancelledError):
+                    display_cancel_message(console)
+                if generated_title:
+                    set_title(f"圣经研读小伙伴 📝 {generated_title}")
+                return generated_title
 
             if not APP_START and args.exit:
                 break
@@ -701,7 +723,7 @@ async def main_async():
                 else:
                     user_request = f".open {chats_path}"
             elif user_request == ".reload":
-                temp_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), "temp")
+                temp_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), f"temp_{config.backend}")
                 last_saved_conversation = os.path.join(temp_dir, "conversation.py")
                 if os.path.isfile(last_saved_conversation):
                     user_request = f".import {temp_dir}"
@@ -729,7 +751,10 @@ async def main_async():
                         print("只接受包含 `conversation.py` 和 `master_plan.md` 的文件夹。")
                         os.chdir(cwd)
                         continue
-                    backup_conversation(messages, master_plan, console)
+                    if config.backup_required:
+                        generated_title = await generate_title()
+                        if generated_title:
+                            backup_conversation(messages, master_plan, console, title=generated_title)
                     config.backup_required = False
                     messages = [{"role": i["role"], "content": i["content"]} for i in eval(readTextFile(file_path)) if i.get("role", "") in ("user", "assistant")]
                     if messages:
@@ -771,7 +796,10 @@ async def main_async():
             # predefined operations with `.` commands
             if user_request in config.action_list:
                 if user_request == ".backup":
-                    backup_conversation(messages, master_plan, console)
+                    if config.backup_required:
+                        generated_title = await generate_title()
+                        if generated_title:
+                            backup_conversation(messages, master_plan, console, title=generated_title)
                     config.backup_required = False
                 elif user_request == ".help":
                     actions = "\n".join([f"- `{k}`: {v}" for k, v in config.action_list.items()])
@@ -1028,10 +1056,15 @@ https://github.com/eliranwong/biblemate
                         write_user_config()
                         display_info(console, f"预设原文字典设定为 `{config.default_lexicon}`", title="设定")
                 elif user_request in (".new", ".exit"):
-                    backup_conversation(messages, master_plan, console) # backup
+                    # backup before reset/exit
+                    if config.backup_required:
+                        generated_title = await generate_title()
+                        if generated_title:
+                            backup_conversation(messages, master_plan, console, title=generated_title)
                     config.backup_required = False
                 # reset
                 if user_request == ".new":
+                    set_title("圣经研读小伙伴 BibleMate AI")
                     user_request = ""
                     master_plan = ""
                     messages = deepcopy(DEFAULT_MESSAGES)
@@ -1126,7 +1159,7 @@ https://github.com/eliranwong/biblemate
 
             # Prompt Engineering
             original_request = user_request
-            if not specified_tool == "@@" and config.prompt_engineering and not user_request in ("[STOP]", "[CONTINUE]"):
+            if ((not specified_tool) or (specified_tool == "get_direct_text_response")) and config.prompt_engineering and not user_request in ("[STOP]", "[CONTINUE]"):
                 improved_prompt_output = ""
                 async def run_prompt_engineering():
                     nonlocal user_request, improved_prompt_output
@@ -1199,7 +1232,8 @@ https://github.com/eliranwong/biblemate
                             console.print(f"错误： {e}\n使用基本简单对话模式 ...\n\n")
                             print(traceback.format_exc())
                         messages = agentmake(messages, system="auto", **AGENTMAKE_CONFIG)
-                messages[-1]["content"] = fix_string(messages[-1]["content"])
+                if messages:
+                    messages[-1]["content"] = fix_string(messages[-1]["content"])
 
             # execute a single tool
             if specified_tool and not specified_tool == "@@" and not specified_prompt:
@@ -1222,20 +1256,22 @@ https://github.com/eliranwong/biblemate
                         await thinking(refine_tool_instruction, "优化工具指示中 ...")
                         if not refined_instruction_output:
                             display_cancel_message(console)
-                            if step == 1:
-                                config.current_prompt = original_request
-                            conversation_broken = True
-                            break
-                        else:
-                            messages[-1]['content'] = refined_instruction
+                            config.current_prompt = original_request
+                            continue
                     except (KeyboardInterrupt, asyncio.CancelledError):
                         display_cancel_message(console)
-                        if step == 1:
-                            config.current_prompt = original_request
-                        conversation_broken = True
-                        break
+                        config.current_prompt = original_request
+                        continue
+                    # review in partner or chat mode
+                    if not config.agent_mode:
+                        display_info(console, "请审阅并确认优化后的指示，或进行任何您需要的更改。", title="检阅")
+                        refined_instruction = await getTextArea(default_entry=refined_instruction, title="检阅：优化后的指示")
+                        if not refined_instruction or refined_instruction == ".exit":
+                            display_cancel_message(console)
+                            continue
+                    messages[-1]['content'] = refined_instruction
                     # display refined instruction
-                    display_info(console, Markdown(messages[-1]['content']), title="Refined Instruction", border_style=get_border_style())
+                    display_info(console, Markdown(refined_instruction), title="优化后的指示", border_style=get_border_style())
                 try:
                     await process_tool(specified_tool, user_request)
                 except (KeyboardInterrupt, asyncio.CancelledError):
@@ -1252,7 +1288,6 @@ https://github.com/eliranwong/biblemate
             # Chat mode
             messages_output = []
             if config.agent_mode is None and not specified_tool == "@@" and not specified_prompt:
-                display_info(console,Markdown(messages[-1]['content']), border_style="none")
                 async def run_chat_mode():
                     nonlocal messages_output, messages, user_request
                     messages_output = agentmake(messages if messages else user_request, system="auto", **AGENTMAKE_CONFIG)
@@ -1272,8 +1307,8 @@ https://github.com/eliranwong/biblemate
                     if messages and messages[-1].get("role", "") == "user":
                         messages = messages[:-1] # remove the last user message
                     continue
-                print()
                 console.print(Markdown(messages[-1]['content']))
+                print()
                 # temporaily save after each step
                 backup_conversation(messages, "")
                 config.backup_required = True
@@ -1466,8 +1501,8 @@ Available tools are: {available_tools}.
                         config.current_prompt = original_request
                     conversation_broken = True
                     break
-                # partner mode
-                if config.agent_mode == False:
+                # partner mode or when request is started with `@@`
+                if not config.agent_mode:
                     display_info(console, "请审阅并确认下一步指示，或进行任何您需要的更改。", title="检阅")
                     next_step_edit = await getTextArea(default_entry=next_step, title="检阅：下一步指示")
                     if not next_step_edit or next_step_edit == ".exit":
@@ -1549,7 +1584,10 @@ Available tools are: {available_tools}.
             # Backup
             if not conversation_broken:
                 print()
-                backup_conversation(messages, master_plan, console)
+                if config.backup_required:
+                    generated_title = await generate_title()
+                    if generated_title:
+                        backup_conversation(messages, master_plan, console, title=generated_title)
                 config.backup_required = False
     
     # back up configurations
